
# Git 笔记

## 创建仓库

    `git init`
为当前工作区设置一个仓库

***

## 提交修改

    `git add filename`
加入暂存区

    `git rm filename`
吧该文件从暂存区中删除

    `git commit filename`
用暂存区更新当前分支  
注意,使用上面的三个东西就可以向仓库中添加文件了

***

## 撤销修改

    `git checkout -- filename`
用来撤销工作区中不合适的修改的.  
而且用的是能找到的,最近最新的修改,解释如下

1. 若暂存区中没有这个文件,则用当前分支的内容来还原
2. 若暂存区中有这个文件,则使用暂存区的内容更新这个文件.

为什么呢?  
因为git在使用的时候,就是"修改工作区--1--add--2--commit"的循环.  
对于情况a,我们处于循环的1阶段,最新的修改存在于版本库中;对于情况2,我们处于循环的2阶段,最新的修改存在于暂存区中.

    `git reset HEAD filename`

用来删除暂存区中的修改  
如果不合适的修改已经加到暂存区中了,使用这个命令先从暂存区中删除,在用上个命令从工作区中删除,就ok

***

## 版本回退

    `git reset --hard commit_ID`

将工作区还原到commit_ID这次提交的状态  
这个命令可以穿梭到未来,或者过去,是版本回退最主要的指令.

    `git log (--pretty==oneline)`

查看每一次提交(commit)历史  
包括版本号,注释

    `git reflog (--pretty=oneline)`

查看使用的每一个命令(包括commit,reset)  
会显示版本号,commit的注释,reset回退的目标版本.

***

## 暂存库与分支

版本库里包含:暂存区(stage)和分支
设计者期望的工作方式是:将所有要提交的文件分别加入暂存区(add),然后一次提交至(commit)分支

## 远程仓库

### 概念

吧本地仓库推送到服务器上,然后所有人就都能看到,复制这个仓库了(如果你同意的话).  
如果想使用Github作为你的远程仓库,免费使用它的服务器的话,首先要设置SSH(传输代码的协议) key.并且把SSH的公钥告诉github.  
这样github就知道当提交文件的时候,是你本人在提交,而不是别人.  
  
>SSH key是每个电脑都不同的么?  

### 使用

>将本地仓库与远程仓库关联起来

    `git remote add origin git@sever's name/path:resp's name.git`
就是这一行

>将本地仓库的修改推送到远程仓库上

    `git push -u origin master`
第一次使用的时候,加上参数u

    `git push origin master`
之后使用的时候,先add,commit,把修改加到仓库中.  
然后使用上面的命令,就把修改推送到远程仓库中了.

***

## 分支

### 概述

分支的意思就是,每个分支都是独立于别的分支的一个复制.  
一个人可以在一个分支里随意做什么事,而不会影响到别的分支.
当在一个分支里做完工作之后,吧两个分支合并就行了.  
而且默认分支叫做"master"

### 用法

    `git branch`
查看当前工作目录有什么分支

    `git branch <name>`
创建`<name>`分支

    `git checkout <name>`
切换到`<name>`分支

    `git checkout -b <name>`
创建并切换到`<name>`分支

    `git branch -d <name>`
删除`<name>`分支

    `git merge <name>`
吧`<name>`分支与当前分支合并(可以说是用`<name>`分支中的内容来更新/补全当前分支.),合并之后还在当前分支  
默认使用快速合并,不能快速合并的时候就会出现冲突(conflict)  
而且`<name>`分支及其中的内容保持不变

### 冲突与解决

>出现冲突的原因是  

要合并的分支与当前分支中包含相互冲突的修改(是什么?),所以git无法决定合并之后的内容是什么  

>解决的思路是  

手动在当前分支中修改内容,替git来决定合并之后的内容应该是什么.

>怎么查看冲突的地方在哪里呢?

+ 使用VScode打开,就有图形化显示了
+ bash中,使用`git status`来查看是哪个文件有冲突,使用`cat <filename>`来查看文件中具体的冲突内容  